@@ -1,7 +1,12 @@
+"""Global Utils.
+
+Only Python Standard Library can be used here.
+"""
 import os
+import datetime
 
 
-class Utils(object):
+class BaseUtils(object):
 
     @staticmethod
     def expandpath(path):
@@ -16,9 +21,9 @@ class Utils(object):
         return real_path
 
     @staticmethod
-    def file_2_sql(file):
+    def file_2_sql(filename):
         """Remove go from sql file for sqlalchemy and pymssql."""
-        with open(file, 'rb') as f:
+        with open(filename, 'r') as f:
             sql_list = f.readlines()
             sql_list = [
                 line
@@ -28,3 +33,8 @@ class Utils(object):
             ]
             sql = "".join(sql_list)
         return sql
+
+    @staticmethod
+    def timestamp():
+        return datetime.datetime.now().strftime('%Y%m%d%H%M%S')
+
