@@ -2,7 +2,7 @@ import subprocess
 import os
 import shutil
 import logging
-import time
+
 
 logger = logging.getLogger(__name__)
 
@@ -20,7 +20,7 @@ class BaseVbox(object):
             if name not in names:
                 return True
             logger.debug("poweroff vm {}.".format(name))
-            cmd = "su {} -c 'vboxmanage controlvm {} poweroff'".format(
+            cmd = "su {} -c '/usr/bin/vboxmanage controlvm {} poweroff'".format(
                 self.username, name)
             logger.debug(cmd)
             process = subprocess.Popen(
@@ -34,25 +34,25 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error(
+            logger.debug(
                 "poweroff vm {} error: {}.".format(name, e.args)
             )
             raise
         else:
             if rc:
-                logger.error(
+                logger.debug(
                     "poweroff vm {} failed with exit_code: {}".format(
                         name, rc)
                 )
                 return False
             else:
-                logger.info("poweroff vm {} succeed.".format(name))
+                logger.debug("poweroff vm {} succeed.".format(name))
                 return True
 
     def delete_vm(self, name):
         try:
             logger.debug("delete vm {}.".format(name))
-            cmd = "su {} -c 'vboxmanage unregistervm {} --delete'".format(
+            cmd = "su {} -c '/usr/bin/vboxmanage unregistervm {} --delete'".format(
                 self.username, name)
             logger.debug(cmd)
             process = subprocess.Popen(
@@ -66,24 +66,24 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error(
+            logger.debug(
                 "delete vm {} error: {}.".format(name, e.args)
             )
             raise
         else:
             if rc:
-                logger.error(
+                logger.debug(
                     "delete vm {} failed with exit_code: {}".format(
                         name, rc)
                 )
                 return False
             else:
-                logger.info("delete vm {} succeed.".format(name))
+                logger.debug("delete vm {} succeed.".format(name))
                 return True
 
     def clone_vm(self, vm, name, basefolder):
         try:
-            cmd = "su {} -c 'vboxmanage clonevm {} --snapshot Clean " \
+            cmd = "su {} -c '/usr/bin/vboxmanage clonevm {} --snapshot Clean " \
                   "--options link --name {} --basefolder {} --register'".format(
                 self.username, vm, name, basefolder)
             logger.debug(cmd)
@@ -98,26 +98,26 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error(
+            logger.debug(
                 "clone vm {} error: {}.".format(name, e.args)
             )
             raise
         else:
             if rc:
-                logger.error(
+                logger.debug(
                     "clone vm {} failed with exit_code: {}".format(
                         name, rc)
                 )
                 return False
             else:
-                logger.info("clone vm {} succeed.".format(name))
+                logger.debug("clone vm {} succeed.".format(name))
                 return True
 
     def attach_storage(self, vm):
         try:
             logger.debug("attach storage for {}.".format(vm))
             cmd = """
-            su {} -c "vboxmanage storageattach {} --storagectl IDE \
+            su {} -c "/usr/bin/vboxmanage storageattach {} --storagectl IDE \
             --port 0 --device 0 --type dvddrive \
             --medium '/usr/share/virtualbox/VBoxGuestAdditions.iso'"
             """.format(self.username, vm)
@@ -133,27 +133,27 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error(
+            logger.debug(
                 "attach storage for vm {} error: {}.".format(vm, e.args)
             )
             raise
         else:
             if rc:
-                logger.error(
+                logger.debug(
                     "attach storage for vm {} failed with exit_code: {}".format(
                         vm, rc)
                 )
                 return False
             else:
-                logger.info("attach storage for vm {} succeed.".format(vm))
+                logger.debug("attach storage for vm {} succeed.".format(vm))
                 return True
 
     def list_vm(self, running=False):
         try:
             if running:
-                cmd = "su {} -c 'vboxmanage list runningvms'".format(self.username)
+                cmd = "su {} -c '/usr/bin/vboxmanage list runningvms'".format(self.username)
             else:
-                cmd = "su {} -c 'vboxmanage list vms'".format(self.username)
+                cmd = "su {} -c '/usr/bin/vboxmanage list vms'".format(self.username)
             logger.debug(cmd)
             process = subprocess.Popen(
                 cmd, shell=True,
@@ -172,7 +172,7 @@ class BaseVbox(object):
                 if line.strip()
             ]
         except Exception as e:
-            logger.error(
+            logger.debug(
                 "list vm for {} error: {}.".format(self.username, e.args)
             )
             raise
@@ -182,7 +182,7 @@ class BaseVbox(object):
 
     def delete_hostonlyif(self, name):
         try:
-            cmd = "su {} -c 'vboxmanage hostonlyif remove {}'".format(self.username, name)
+            cmd = "su {} -c '/usr/bin/vboxmanage hostonlyif remove {}'".format(self.username, name)
             logger.debug(cmd)
             process = subprocess.Popen(
                 cmd, shell=True,
@@ -195,14 +195,14 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error(
+            logger.debug(
                 "delete virtualbox hostonlyif {} error: {}.".format(name,
                                                                     e.args)
             )
             raise
         else:
             if rc:
-                logger.error(
+                logger.debug(
                     "delete hostonlyif {} failed with exit_code: {}".format(
                         name, rc)
                 )
@@ -214,7 +214,7 @@ class BaseVbox(object):
     def create_hostonlyif(self):
         try:
             logger.info("create networks.")
-            cmd = "su {} -c 'vboxmanage hostonlyif create'".format(self.username)
+            cmd = "su {} -c '/usr/bin/vboxmanage hostonlyif create'".format(self.username)
             logger.debug(cmd)
             process = subprocess.Popen(
                 cmd, shell=True,
@@ -227,11 +227,11 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error("create hostonlyif error: {}.".format(e.args))
+            logger.debug("create hostonlyif error: {}.".format(e.args))
             raise
         else:
             if rc:
-                logger.error(
+                logger.debug(
                     "create hostonlyif failed with exit_code: {}".format(rc)
                 )
                 return False
@@ -242,7 +242,7 @@ class BaseVbox(object):
     def modify_hostonlyif(self, name, gateway, netmask):
         try:
             logger.info("modify networks.")
-            cmd = "su {} -c 'vboxmanage hostonlyif ipconfig {} --ip {} " \
+            cmd = "su {} -c '/usr/bin/vboxmanage hostonlyif ipconfig {} --ip {} " \
                   "--netmask {}'".format(self.username, name, gateway, netmask)
             logger.debug(cmd)
             process = subprocess.Popen(
@@ -256,12 +256,12 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error("modify hostonlyif {} error: {}.".format(name,
+            logger.debug("modify hostonlyif {} error: {}.".format(name,
                                                                   e.args))
             raise
         else:
             if rc:
-                logger.error(
+                logger.debug(
                     "modify hostonlyif {} failed with exit_code: {}".format(
                         name, rc)
                 )
@@ -277,7 +277,7 @@ class BaseVbox(object):
                 shutil.rmtree(
                     os.path.join(basefolder, name)
                 )
-            cmd = "su {} -c 'vboxmanage import {} --vsys 0 --vmname {} " \
+            cmd = "su {} -c '/usr/bin/vboxmanage import {} --vsys 0 --vmname {} " \
                   "--basefolder {}'".format(self.username,
                                             ova, name, basefolder)
             logger.debug(cmd)
@@ -292,11 +292,11 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error("import ova {} error: {}.".format(ova, e.args))
+            logger.debug("import ova {} error: {}.".format(ova, e.args))
             raise
         else:
             if rc:
-                logger.error(
+                logger.debug(
                     "import ova {} failed with exit_code: {}".format(
                         ova, rc)
                 )
@@ -308,7 +308,7 @@ class BaseVbox(object):
     def modify_vm(self, vm, hostonlyif, cpu, memory):
         try:
             logger.debug("modify base vm {}".format(vm))
-            cmd = "su {} -c 'vboxmanage modifyvm {} --nic1 hostonly " \
+            cmd = "su {} -c '/usr/bin/vboxmanage modifyvm {} --nic1 hostonly " \
                   "--hostonlyadapter1 {} " \
                   "--cpus {} --memory {}'".format(
                 self.username, vm, hostonlyif, cpu, memory)
@@ -324,11 +324,11 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error("modify vm {} error: {}.".format(vm, e.args))
+            logger.debug("modify vm {} error: {}.".format(vm, e.args))
             raise
         else:
             if rc:
-                logger.error(
+                logger.debug(
                     "modify vm {} failed with exit_code: {}".format(
                         vm, rc)
                 )
@@ -340,7 +340,7 @@ class BaseVbox(object):
     def take_snapshot(self, vm, name):
         try:
             logger.debug("take snapshot for {}".format(vm))
-            cmd = "su {} -c 'vboxmanage snapshot {} take {} --live " \
+            cmd = "su {} -c '/usr/bin/vboxmanage snapshot {} take {} --live " \
                   "--pause'".format(self.username, vm, name)
             logger.debug(cmd)
             process = subprocess.Popen(
@@ -354,7 +354,7 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error("take snapshot for {} error: {}.".format(vm, e.args))
+            logger.debug("take snapshot for {} error: {}.".format(vm, e.args))
             raise
         else:
             if rc:
@@ -370,7 +370,7 @@ class BaseVbox(object):
     def restore_snapshot(self, vm, name):
         try:
             logger.debug("restore snapshot for {}".format(vm))
-            cmd = "su {} -c 'vboxmanage snapshot {} restore {}".format(
+            cmd = "su {} -c '/usr/bin/vboxmanage snapshot {} restore {}'".format(
                 self.username, vm, name
             )
             logger.debug(cmd)
@@ -385,7 +385,7 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error(
+            logger.debug(
                 "restore snapshot for {} error: {}.".format(vm, e.args)
             )
             raise
@@ -403,7 +403,7 @@ class BaseVbox(object):
     def delete_snapshot(self, vm, name):
         try:
             logger.debug("delete snapshot for {}".format(vm))
-            cmd = "su {} -c 'vboxmanage snapshot {} delete {}".format(
+            cmd = "su {} -c '/usr/bin/vboxmanage snapshot {} delete {}'".format(
                 self.username, vm, name)
             logger.debug(cmd)
             process = subprocess.Popen(
@@ -417,7 +417,7 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error(
+            logger.debug(
                 "delete snapshot for {} error: {}.".format(vm, e.args)
             )
             raise
@@ -435,7 +435,7 @@ class BaseVbox(object):
     def start_vm(self, vm):
         try:
             logger.debug("start vm {}".format(vm))
-            cmd = "su {} -c 'vboxmanage startvm {} --type headless'".format(
+            cmd = "su {} -c '/usr/bin/vboxmanage startvm {} --type headless'".format(
                 self.username, vm)
             logger.debug(cmd)
             process = subprocess.Popen(
@@ -449,7 +449,7 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error("start vm {} error: {}.".format(vm, e.args))
+            logger.debug("start vm {} error: {}.".format(vm, e.args))
             raise
         else:
             if rc:
@@ -465,15 +465,15 @@ class BaseVbox(object):
     def remove_uninst(self, vm):
         """
         Linux remove uninst:
-        $ vboxmanage guestcontrol Linux64 run --username root --password pw
-         /bin/bash -- -l -c  '/bin/mount /dev/cdrom1 /media/cdrom'
-        $ vboxmanage guestcontrol Linux64 run --username user --password pw
-         /bin/bash -- -l -c  'cd /media/cdrom; sh VBoxLinuxAdditions.run uninstall'
+        $ /usr/bin/vboxmanage guestcontrol Linux64 run --username root --password pw
+         --exe /bin/bash -- -l -c  '/bin/mount /dev/cdrom1 /media/cdrom'
+        $ /usr/bin/vboxmanage guestcontrol Linux64 run --username user --password pw
+         --exe /bin/bash -- -l -c  'cd /media/cdrom; sh VBoxLinuxAdditions.run uninstall'
         """
         try:
             logger.debug("remove uninst for {}".format(vm))
             cmd = """
-            su {} -c "vboxmanage guestcontrol {} --username 'Administrator' \
+            su {} -c "/usr/bin/vboxmanage guestcontrol {} --username 'Administrator' \
             run --exe \
             'C:\\Program Files\\Oracle\\VirtualBox Guest Additions\\uninst.exe' \
              -- uninst.exe /S"
@@ -490,7 +490,7 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error("remove uninst {} error: {}.".format(vm, e.args))
+            logger.debug("remove uninst {} error: {}.".format(vm, e.args))
             raise
         else:
             if rc:
@@ -506,7 +506,7 @@ class BaseVbox(object):
         try:
             logger.debug("setup ip address for {}".format(vm))
             cmd = """
-             su {} -c "vboxmanage guestcontrol {} --username 'Administrator' \
+             su {} -c "/usr/bin/vboxmanage guestcontrol {} --username 'Administrator' \
              run --exe 'C:\\Windows\\system32\\cmd.exe' -- \
              cmd.exe /c netsh interface ip set address \
              name='Local Area Connection' static {} {} {} 1"
@@ -525,7 +525,7 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error("setup ip {} error: {}.".format(vm, e.args))
+            logger.debug("setup ip {} error: {}.".format(vm, e.args))
             raise
         else:
             if rc:
@@ -541,7 +541,7 @@ class BaseVbox(object):
         try:
             logger.debug("setup dns for {}.".format(vm))
             cmd = """
-            su {} -c "vboxmanage guestcontrol {} --username 'Administrator' \
+            su {} -c "/usr/bin/vboxmanage guestcontrol {} --username 'Administrator' \
             run --exe 'C:\\Windows\\system32\\cmd.exe' -- \
             cmd.exe /c netsh interface ip set dns \
             name='Local Area Connection' static {}"
@@ -558,11 +558,11 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error("setup dns {} error: {}.".format(vm, e.args))
+            logger.debug("setup dns {} error: {}.".format(vm, e.args))
             raise
         else:
             if rc:
-                logger.error(
+                logger.debug(
                     "setup dns {} failed with exit_code: {}".format(vm, rc)
                 )
                 return False
@@ -570,13 +570,44 @@ class BaseVbox(object):
                 logger.debug("setup dns {} succeed.".format(vm))
                 return True
 
+    def register_vm(self, path, name):
+        """ register will register vbox to vboxmanage."""
+        try:
+            logger.info("register vm {}.".format(name))
+            cmd = """
+            su {} -c "/usr/bin/vboxmanage registervm {}/{}/{}.vbox
+            """.format(self.username, path, name, name)
+            logger.debug(cmd)
+            process = subprocess.Popen(
+                cmd, shell=True,
+                stdout=subprocess.PIPE, stderr=subprocess.PIPE
+            )
+            output, error = process.communicate()
+            output = output.decode("utf-8")
+            error = error.decode("utf-8")
+            logger.debug("output: {}".format(output))
+            logger.debug("error: {}".format(error))
+            rc = process.returncode
+        except Exception as e:
+            logger.debug("register vm {} error: {}.".format(name, e.args))
+            raise
+        else:
+            if rc:
+                logger.debug(
+                    "register vm {} failed with exit_code: {}".format(name, rc)
+                )
+                return False
+            else:
+                logger.debug("register vm {} succeed.".format(name))
+                return True
+
     def install_license(self, vm, lic):
         try:
             logger.debug("install license for {}.".format(vm))
             cmd = """
-            su {} -c "vboxmanage guestcontrol {} --username 'Administrator' \
+            su {} -c "/usr/bin/vboxmanage guestcontrol {} --username 'Administrator' \
             run --exe 'C:\\Windows\\system32\\cmd.exe' -- \
-            cmd.exe /c slmgr.vbs -ipk {}"
+            cmd.exe /c cscript slmgr.vbs -ipk {}"
             """.format(self.username, vm, lic)
             logger.debug(cmd)
             process = subprocess.Popen(
@@ -590,11 +621,11 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error("install license {} error: {}.".format(vm, e.args))
+            logger.debug("install license {} error: {}.".format(vm, e.args))
             raise
         else:
             if rc:
-                logger.error(
+                logger.debug(
                     "install license {} failed with exit_code: {}".format(vm, rc)
                 )
                 return False
@@ -606,9 +637,9 @@ class BaseVbox(object):
         try:
             logger.debug("active license for {}.".format(vm))
             cmd = """
-            su {} -c "vboxmanage guestcontrol {} --username 'Administrator' 
+            su {} -c "/usr/bin/vboxmanage guestcontrol {} --username 'Administrator' \
             run --exe 'C:\\Windows\\system32\\cmd.exe' -- \
-            cmd.exe /c slmgr.vbs -ato"
+            cmd.exe /c cscript slmgr.vbs -ato"
             """.format(self.username, vm)
             logger.debug(cmd)
             process = subprocess.Popen(
@@ -622,11 +653,11 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error("active license {} error: {}.".format(vm, e.args))
+            logger.debug("active license {} error: {}.".format(vm, e.args))
             raise
         else:
             if rc:
-                logger.error(
+                logger.debug(
                     "active license {} failed with exit_code: {}".format(vm, rc)
                 )
                 return False
@@ -634,10 +665,53 @@ class BaseVbox(object):
                 logger.debug("active license {} succeed.".format(vm))
                 return True
 
+    def check_license(self, vm):
+        """
+        check windows license status.
+        :param vm:
+        :return:
+
+        slmgr.vbs \dli
+        OR
+        slmgr.vbs \dlv
+        """
+        try:
+            logger.debug("check license for {}.".format(vm))
+            cmd = """
+            su {} -c "/usr/bin/vboxmanage guestcontrol {} --username 'Administrator' \
+            run --exe 'C:\\Windows\\system32\\cmd.exe' -- \
+            cmd.exe /c cscript slmgr.vbs -dli"
+            """.format(self.username, vm)
+            logger.debug(cmd)
+            process = subprocess.Popen(
+                cmd, shell=True,
+                stdout=subprocess.PIPE, stderr=subprocess.PIPE
+            )
+            output, error = process.communicate()
+            output = output.decode("utf-8")
+            error = error.decode("utf-8")
+            logger.debug("output: {}".format(output))
+            logger.debug("error: {}".format(error))
+            rc = 127
+            if "License Status: Licensed" in output:
+                rc = 0
+        except Exception as e:
+            logger.debug("active license {} error: {}.".format(vm, e.args))
+            raise
+        else:
+            if rc:
+                logger.debug(
+                    "{} not licensed.".format(vm)
+                )
+                return False
+            else:
+                logger.debug("{} licensed.".format(vm))
+                return True
+
     def win_cmd(self, vm, cmd, user='Administrator'):
         try:
             cmd = """
-             su {} -c "vboxmanage guestcontrol {} --username {} \
+             su {} -c "/usr/bin/vboxmanage guestcontrol {} --username {} \
              run --exe 'C:\\Windows\\system32\\cmd.exe' -- \
              cmd.exe /c {}
              """.format(
@@ -655,7 +729,7 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error("run cmd on {} error: {}.".format(vm, e.args))
+            logger.debug("run cmd on {} error: {}.".format(vm, e.args))
             raise
         else:
             if rc:
@@ -671,8 +745,8 @@ class BaseVbox(object):
         try:
             logger.debug("shell: {}".format(shell))
             cmd = """
-            su {} -c "vboxmanage guestcontrol {} --username {} \
-            --password {} run /bin/bash -- -l -c '{}'"
+            su {} -c "/usr/bin/vboxmanage guestcontrol {} --username {} \
+            --password {} run --exe /bin/bash -- -l -c '{}'"
             """.format(self.username, vm, username, password, shell)
             logger.debug(cmd)
             process = subprocess.Popen(
@@ -686,7 +760,7 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error(
+            logger.debug(
                 "run linux command on {} error: {}.".format(vm, e.args)
             )
             raise
@@ -704,11 +778,11 @@ class BaseVbox(object):
         try:
             if parent:
                 cmd = """
-                su {} -c "vboxmanage list hdds | grep '^Parent UUID:'"
+                su {} -c "/usr/bin/vboxmanage list hdds | grep '^Parent UUID:'"
                 """.format(self.username)
             else:
                 cmd = """
-                su {} -c "vboxmanage list hdds | grep '^UUID:'"
+                su {} -c "/usr/bin/vboxmanage list hdds | grep '^UUID:'"
                 """.format(self.username)
 
             logger.debug(cmd)
@@ -728,7 +802,7 @@ class BaseVbox(object):
                 if "-" in line.strip()
             ]
         except Exception as e:
-            logger.error(
+            logger.debug(
                 "list hdds for {} error: {}.".format(self.username, e.args)
             )
             raise
@@ -738,7 +812,7 @@ class BaseVbox(object):
     def delete_hdd(self, hdd):
         try:
             logger.debug("delete hdd {}.".format(hdd))
-            cmd = "su {} -c 'vboxmanage closemedium disk {} --delete'".format(
+            cmd = "su {} -c '/usr/bin/vboxmanage closemedium disk {} --delete'".format(
                 self.username, hdd)
             logger.debug(cmd)
             process = subprocess.Popen(
@@ -752,13 +826,13 @@ class BaseVbox(object):
             logger.debug("error: {}".format(error))
             rc = process.returncode
         except Exception as e:
-            logger.error(
+            logger.debug(
                 "delete hdd {} error: {}.".format(hdd, e.args)
             )
             raise
         else:
             if rc:
-                logger.error(
+                logger.debug(
                     "delete hdd {} failed with exit_code: {}".format(
                         hdd, rc)
                 )
@@ -808,7 +882,7 @@ class BaseVbox(object):
             if os.path.isdir(basefolder):
                 shutil.rmtree(basefolder, ignore_errors=True)
         except Exception as e:
-            logger.error("purge vm error: {}.".format(e.args))
+            logger.debug("purge vm error: {}.".format(e.args))
             raise
         else:
             return True
@@ -823,7 +897,7 @@ class BaseVbox(object):
                         self.delete_hdd(hdd)
                 hdds = self.list_hdd()
         except Exception as e:
-            logger.error("purge hdd error: {}.".format(e.args))
+            logger.debug("purge hdd error: {}.".format(e.args))
             raise
         else:
             return True
